* specification 4 = Pre-period FGLS ****************************************************************************************
clear all
set more off  
cd "N:\_Personal Work Folder\Baker\Volatility Adjustment Papers\Extended Data"
use priceseries_extended, clear

gen estimation = (date >= d(09aug2005) && date <= d(08aug2006))
summ estimation
scalar n = r(sum)
gen testperiod = (date >= d(09aug2006) && date <= d(08aug2007))

** create date indicator variables 

forval i = 1660/1910 {
   gen date`i' = 1 in `i'
   }

* Set up vix to be on a daily level
gen vix_daily = (vix/100)/(250^0.5)
gen v_vix_daily = vix_daily^2

* Create the FGLS code	
program define fgls
   reg `1' `2' if `3'==1 
     predict double e, resid
   gen e2 = (e)^2
   reg e2 v_vix_daily if `3'==1, nocons
   predict double e2_hat
  gen weight = 1/e2_hat
   reg `1' `2' if `3'==1 [aw=weight]
   drop e e2 e2_hat weight
   mat b0 = e(b)
  local i=1 
  local check = 0
   while `check' != 1 { 
   predict double e_2, resid
   gen e2_2 = e_2^2
  reg e2_2 v_vix_daily if `3'==1, nocons
  predict double e2_2hat
  gen weight2 = 1/e2_2hat
  reg `1' `2' if `3'==1 [aw=weight2]
  drop e_2 e2_2  weight2
  mat b`i' = e(b)
  local j = `i' - 1
  mat diff = b`i' - b`j'
  mat list diff
  forval k = 1/4 {
  scalar a1`k' = diff[1,`k']
  }
  if abs(scalar(a11)) < 0.0001 & abs(scalar(a12)) < 0.0001{
  local check = 1
  display "Converged after iteration number " `i'
  } 
  else { 
  drop e2_2hat
  local i = `i' + 1 
  display `i' 
  } 
     } 
  end
 
 
foreach y of varlist mmm-dis{
	foreach x in _arith _log{
	
fgls `y'`x' spxt`x' estimation
predict `y'`x'_hat
predict `y'`x'_ar, resid
gen `y'`x'_se = (e2_2hat)^0.5
gen `y'`x'_tstat = `y'`x'_ar / `y'`x'_se
drop e2_2hat
}
}

keep date *_tstat
keep if (date >= d(09aug2006) && date <= d(08aug2007))

foreach var of varlist mmm_arith_tstat - dis_log_tstat{
	gen ind_2t`var' = (ttail(n,abs(`var'))<=.025)
	gen ind_1t`var' = (ttail(n,abs(`var'))<=.05)
	}

keep date ind* 

reshape long ind_2 ind_1, i(date) j(security) string
collapse (mean) ind_2 ind_1, by(security)

export excel using "Results One Year Prior.xlsx", sheet(Pre-Period FGLS) sheetmodify firstrow(variables) cell(A1) 
