clear all
set more off

cd "C:\Users\Andrew\Documents\Personal Folders\Volatility Adjustment Papers\SP500 Analysis"
use SP500_stock, clear
gen log_return = ln(ret+1)
gen log_sp = ln(sprtrn+1)

** drop these stocks because they don't have trading data through one year prior to crisis until end of crisis
foreach x in 51196 51539 52396 52417 52418 53296 8283 41998 14097 10604 29941 21651 21472 29875 5849 21301{
	drop if permco == `x'
}
** drop these stocks because they have two listed share classes and we only need return for one
foreach x in 60871R10 21036P20 57978010 85590A20 52605710 12485710{
	drop if cusip == "`x'"
}
keep date permco log_return log_sp
cd "C:\Users\Andrew\Documents\Personal Folders\Volatility Adjustment Papers\Extended Data"
merge m:1 date using priceseries, keepusing(vix) keep(match master) nogen

sort permco date

cd "C:\Users\Andrew\Documents\Personal Folders\Volatility Adjustment Papers\SP500 Analysis"
by permco: gen time = _n
foreach x in pp is roll{
	foreach y in ols vixols fgls sq{
		gen ar_`x'_`y' = 0
		gen se_`x'_`y' = 0
		gen sig_`x'_`y' = 0
	}
}
save Sp500_master_data, replace 

cd "C:\Users\Andrew\Documents\Personal Folders\Volatility Adjustment Papers\SP500 Analysis"
use master_data, replace
** Pre Period OLS

summ log_return if date >= d(09aug2006) & date <= d(08aug2007)
scalar n = r(N)
levelsof permco, local(sec)
global sample = "pp_ols"
	foreach x of local sec{
		reg log_return log_sp if date >= d(09aug2006) & date <= d(08aug2007) & permco == `x'
		predict resid, resid
			replace ar_$sample = resid if permco == `x'
			replace se_$sample = e(rmse) if permco == `x'
		gen t = resid/e(rmse)
			replace sig_$sample = 1 if ttail(n,abs(t))<=.025 & permco == `x'
		drop resid t
	}
save $sample, replace

** In Sample OLS
summ log_return if date >= d(09aug2007) & date <= d(23feb2009)
scalar n = r(N)
levelsof permco, local(sec)
global sample = "is_ols"
	foreach x of local sec{
		reg log_return log_sp if date >= d(09aug2007) & date <= d(23feb2009) & permco == `x'
		predict resid, resid
			replace ar_$sample = resid if permco == `x'
			replace se_$sample = e(rmse) if permco == `x'
		gen t = resid/e(rmse)
			replace sig_$sample = 1 if ttail(n,abs(t))<=.025 & permco == `x'
		drop resid t
	}
save $sample, replace

** Rolling 250 OLS
levelsof permco, local(sec)
global sample = "roll_ols"
	foreach x of local sec{
		summ time if date == d(09aug2007) & permco == `x'
			local start = r(mean)
		summ time if date == d(23feb2009) & permco == `x'
			local end = r(mean)
		forval i = `start'/`end'{
			reg log_return log_sp if permco == `x' & time >=  (`i'-250) & time <= (`i'-1)
			predict resid, resid
				replace ar_$sample = resid if permco == `x' & time == `i'
				replace se_$sample = e(rmse) if permco == `x' & time == `i'
			gen t = resid/e(rmse)
				replace sig_$sample = 1 if ttail(250,abs(t))<=.025 & permco == `x' & time == `i'	
			drop resid t 
		}
	}
}

save $sample, replace

** Pre Period OLS with vix adjustment

summ log_return if date >= d(09aug2006) & date <= d(08aug2007)
scalar n = r(N)
summ vix if date >= d(09aug2006) & date <= d(08aug2007)
scalar samplevix = r(mean)
levelsof permco, local(sec)
global sample = "pp_vixols"
	foreach x of local sec{
		reg log_return log_sp if date >= d(09aug2006) & date <= d(08aug2007) & permco == `x'
		predict resid, resid
			replace ar_$sample = resid if permco == `x'
		gen replace_factor = vix/samplevix	
			replace se_$sample = e(rmse)*replace_factor if permco == `x'
		gen t = resid/(e(rmse)*replace_factor)
			replace sig_$sample = 1 if ttail(n,abs(t))<=.025 & permco == `x'
		drop resid t replace_factor
	}
save $sample, replace

** In-Sample OLS with vix adjustment

summ log_return if date >= d(09aug2007) & date <= d(23feb2009)
scalar n = r(N)
summ vix if date >= d(09aug2007) & date <= d(23feb2009)
scalar samplevix = r(mean)
levelsof permco, local(sec)
global sample = "is_vixols"
	foreach x of local sec{
		reg log_return log_sp if date >= d(09aug2007) & date <= d(23feb2009) & permco == `x'
		predict resid, resid
			replace ar_$sample = resid if permco == `x'
		gen replace_factor = vix/samplevix	
			replace se_$sample = e(rmse)*replace_factor if permco == `x'
		gen t = resid/(e(rmse)*replace_factor)
			replace sig_$sample = 1 if ttail(n,abs(t))<=.025 & permco == `x'
		drop resid t replace_factor
	}
save is_vixols, replace

** Rolling sample OLS With Vix Adjustment

levelsof permco, local(sec)
global sample = "roll_vixols"

	foreach x of local sec{
		summ time if date == d(09aug2007) & permco == `x'
			local start = r(mean)
		summ time if date == d(23feb2009) & permco == `x'
			local end = r(mean)
		forval i = `start'/`end'{		
		summ vix if time >= (`i'-250) & time <= (`i'-1) & permco == `x'
			scalar vixadj = r(mean)
		reg log_return log_sp if time >= (`i'-250) & time <= (`i'-1) & permco == `x'
		predict resid, resid
			replace ar_$sample = resid if permco == `x' & time == `i'
			replace se_$sample = (e(rmse)*vix/vixadj) if permco == `x' & time == `i'
		gen t = resid/(e(rmse)*vix/vixadj)
			replace sig_$sample = 1 if ttail(250,abs(t))<=.025 & permco == `x' & time == `i'
		drop resid t 
		}
	}

save $sample, replace

** Pre-period FGLS

gen vix_daily = (vix/100)/(250^0.5)
gen v_vix_daily = vix_daily^2

gen estimation = (date >= d(09aug2006) & date <= d(08aug2007))
* Create the FGLS code	
program define fgls
   reg `1' `2' if `3'==1 & `4'==`5'
     predict double e, resid
   gen e2 = (e)^2
   reg e2 v_vix_daily if `3'==1 & `4'==`5', nocons
   predict double e2_hat
  gen weight = 1/e2_hat
   reg `1' `2' if `3'==1 & `4'==`5' [aw=weight]
   drop e e2 e2_hat weight
   mat b0 = e(b)
  local i=1 
  local check = 0
   while `check' != 1 { 
   predict double e_2, resid
   gen e2_2 = e_2^2
  reg e2_2 v_vix_daily if `3'==1 & `4'==`5', nocons
  predict double e2_2hat
  gen weight2 = 1/e2_2hat
  reg `1' `2' if `3'==1 & `4'==`5' [aw=weight2]
  drop e_2 e2_2  weight2
  mat b`i' = e(b)
  local j = `i' - 1
  mat diff = b`i' - b`j'
  mat list diff
  forval k = 1/4 {
  scalar a1`k' = diff[1,`k']
  }
  if abs(scalar(a11)) < 0.0001 & abs(scalar(a12)) < 0.0001{
  local check = 1
  display "Converged after iteration number " `i'
  } 
  else { 
  drop e2_2hat
  local i = `i' + 1 
  display `i' 
  } 
     } 
  end
 
levelsof permco, local(sec)
global sample = "pp_fgls" 
summ log_return if date >= d(09aug2006) & date <= d(08aug2007)
scalar n = r(N) 
	foreach x of local sec{
		fgls log_return log_sp estimation permco `x'
		predict resid, resid
		gen se = (e2_2hat)^0.5
			replace ar_$sample = resid if permco == `x'
			replace se_$sample = se if permco == `x'
		gen t = resid/se
			replace sig_$sample = 1 if ttail(n,abs(t))<=.025 & permco == `x' 
		drop e2_2hat se resid t
	}

save $sample, replace
** In Sample FGLS
gen testperiod = (date >= d(09aug2007) & date <= d(23feb2009))
levelsof permco, local(sec)
global sample = "is_fgls" 
summ log_return if date >= d(09aug2007) & date <= d(23feb2009)
scalar n = r(N) 
	foreach x of local sec{
		fgls log_return log_sp testperiod permco `x'
		predict resid, resid
		gen se = (e2_2hat)^0.5
			replace ar_$sample = resid if permco == `x'
			replace se_$sample = se if permco == `x'
		gen t = resid/se
			replace sig_$sample = 1 if ttail(n,abs(t))<=.025 & permco == `x' 
		drop e2_2hat se resid t
	}
	
save $sample, replace

** Rolling 250 FGLS
* Create the rolling FGLS code	
program define fgls_2
   reg `1' `2' if time >=`3' & time<=`4' & `5'==`6'
     predict double e, resid
   gen e2 = (e)^2
   reg e2 v_vix_daily if time >=`3' & time<=`4' & `5'==`6', nocons
   predict double e2_hat
  gen weight = 1/e2_hat
   reg `1' `2' if time >=`3' & time<=`4' & `5'==`6' [aw=weight]
   drop e e2 e2_hat weight
   mat b0 = e(b)
  local i=1 
  local check = 0
   while `check' != 1 { 
   predict double e_2, resid
   gen e2_2 = e_2^2
  reg e2_2 v_vix_daily if time >=`3' & time<=`4' & `5'==`6', nocons
  predict double e2_2hat
  gen weight2 = 1/e2_2hat
  reg `1' `2' if time >=`3' & time<=`4' & `5'==`6' [aw=weight2]
  drop e_2 e2_2  weight2
  mat b`i' = e(b)
  local j = `i' - 1
  mat diff = b`i' - b`j'
  mat list diff
  forval k = 1/4 {
  scalar a1`k' = diff[1,`k']
  }
  if abs(scalar(a11)) < 0.0001 & abs(scalar(a12)) < 0.0001{
  local check = 1
  display "Converged after iteration number " `i'
  } 
  else { 
  drop e2_2hat
  local i = `i' + 1 
  display `i' 
  } 
     } 
  end
  
levelsof permco, local(sec)
global sample = "roll_fgls"
	foreach x of local sec{
		summ time if date == d(09aug2007) & permco == `x'
			local start = r(mean)
		summ time if date == d(23feb2009) & permco == `x'
			local end = r(mean)
		forval i = `start'/`end'{
		fgls_2 log_return log_sp (`i'-250) (`i'-1) permco `x'
		predict resid, resid
		gen se = (e2_2hat)^0.5
			replace ar_$sample = resid if permco == `x' & time == `i'
			replace se_$sample = se if permco == `x' & time == `i'
		gen t = resid/se
			replace sig_$sample = 1 if ttail(250,abs(t))<=.025 & permco == `x' & time == `i'
		drop e2_2hat se resid t	
			
	}
}
save $sample, replace

levelsof permco, local(sec)
global sample = "pp_sq"
	foreach x of local sec{
		reg log_return log_sp if date >= d(09aug2006) & date <= d(08aug2007) & permco == `x'
		predict resid, resid
			replace ar_$sample = resid if permco == `x'
			_pctile resid if date >= d(09aug2006) & date <= d(08aug2007) & permco == `x',  p(2.5,97.5) altdef
			replace sig_$sample = 1 if (resid <= r(r1) | resid >= r(r2)) & permco == `x'
		drop resid
	}
save $sample, replace

levelsof permco, local(sec)
global sample = "is_sq"
	foreach x of local sec{
		reg log_return log_sp if date >= d(09aug2007) & date <= d(23feb2009) & permco == `x'
		predict resid, resid
			replace ar_$sample = resid if permco == `x'
			_pctile resid if date >= d(09aug2007) & date <= d(23feb2009) & permco == `x',  p(2.5,97.5) altdef
			replace sig_$sample = 1 if (resid <= r(r1) | resid >= r(r2)) & permco == `x'
		drop resid
	}

save $sample, replace

*** Rolling 240 SQ
levelsof permco, local(sec)
global sample = "roll_sq"
	foreach x of local sec{
		summ time if date == d(09aug2007) & permco == `x'
			local start = r(mean)
		summ time if date == d(23feb2009) & permco == `x'
			local end = r(mean)
	forval i = `start'/`end'{
		reg log_return log_sp if time >= (`i'-240) & time <= (`i'-1) & permco == `x'
		predict resid, resid
			replace ar_$sample = resid if permco == `x' & time == `i'
			_pctile resid if time >= (`i'-240) & time <= (`i'-1) & permco == `x', p(2.5,97.5) altdef
			replace sig_$sample = 1 if (resid <= r(r1) | resid >= r(r2)) & permco == `x' & time == `i'
		drop resid
	}
}

save $sample, replace
