* Pre Period OLS
clear all
set more off
cd "C:\Users\Andrew\Documents\Personal Folders\Volatility Adjustment Papers"
use priceseries, clear

gen estimation = (date >= d(09aug2006) && date <= d(08aug2007))
summ estimation
scalar n = r(sum)
gen testperiod = (date >= d(09aug2007) && date <= d(23feb2009))

foreach var of varlist mmm-dis{
		reg `var'_log spxt_log if estimation == 1
		predict `var'_ols_pp_ar, resid
		gen `var'_ols_pp_se = e(rmse)
		gen `var'_ols_pp_sig = (ttail(n,abs(`var'_ols_pp_ar/e(rmse)))<=.025)
}

** Rollings OLS
gen time = _n
summ time if date == d(09aug2007)
	local start = r(mean)
summ time if date == d(23feb2009)
	local end = r(mean)
	
forval i = `start'/`end'{
	qui gen date`i' = 0
		local k = `i' - 250
		local j = `i' - 1
			forval m = `k'/`j'{
				qui replace date`i' = 1 in `m'
			}
}

foreach var of varlist mmm-dis{
		gen `var'_ols_roll_ar = .
		gen `var'_ols_roll_se = .
		gen `var'_ols_roll_sig = .
	}

forval i = `start'/`end'{
	foreach var of varlist mmm-dis{
			reg `var'_log spxt_log if date`i' == 1
			predict ar`i', resid
				replace `var'_ols_roll_ar = ar`i' in `i'
				replace `var'_ols_roll_se = e(rmse) in `i'
				replace `var'_ols_roll_sig = (ttail(250,abs(`var'_ols_roll_ar/e(rmse)))<=.025) in `i'
		drop ar`i' 	
	}
}

** Pre Period OLS with VIX adj
summ vix if estimation == 1
	local est_vix = r(mean)

gen scale_factor = vix/`est_vix'
	
foreach var of varlist mmm-dis{
		reg `var'_log spxt_log if estimation == 1
		predict `var'_vixols_pp_ar, resid
		gen `var'_vixols_pp_se = e(rmse)*scale_factor
		gen `var'_vixols_pp_sig = (ttail(n,abs(`var'_vixols_pp_ar/(e(rmse)*scale_factor)))<=.025)
}

** Rolling Window OLS with VIX adj

foreach var of varlist mmm-dis{
		gen `var'_vixols_roll_ar = .
		gen `var'_vixols_roll_se = .
		gen `var'_vixols_roll_sig = .
	}

forval i = `start'/`end'{
	foreach var of varlist mmm-dis{
			reg `var'_log spxt_log if date`i' == 1
			predict ar`i', resid
			summ vix if date`i' == 1	
				local est_vix = r(mean)
			replace `var'_vixols_roll_ar = ar`i' in `i'
				replace `var'_vixols_roll_se = e(rmse)*(vix/`est_vix') in `i'
				replace `var'_vixols_roll_sig = (ttail(250,abs(`var'_vixols_roll_ar/`var'_vixols_roll_se))<=.025) in `i'
		drop ar`i' 	
	}
}

** pp fgls
gen vix_daily = (vix/100)/(250^0.5)
gen v_vix_daily = vix_daily^2

* Create the FGLS code	
program define fgls
   reg `1' `2' if `3'==1 
     predict double e, resid
   gen e2 = (e)^2
   reg e2 v_vix_daily if `3'==1, nocons
   predict double e2_hat
  gen weight = 1/e2_hat
   reg `1' `2' if `3'==1 [aw=weight]
   drop e e2 e2_hat weight
   mat b0 = e(b)
  local i=1 
  local check = 0
   while `check' != 1 { 
   predict double e_2, resid
   gen e2_2 = e_2^2
  reg e2_2 v_vix_daily if `3'==1, nocons
  predict double e2_2hat
  gen weight2 = 1/e2_2hat
  reg `1' `2' if `3'==1 [aw=weight2]
  drop e_2 e2_2  weight2
  mat b`i' = e(b)
  local j = `i' - 1
  mat diff = b`i' - b`j'
  mat list diff
  forval k = 1/4 {
  scalar a1`k' = diff[1,`k']
  }
  if abs(scalar(a11)) < 0.0001 & abs(scalar(a12)) < 0.0001{
  local check = 1
  display "Converged after iteration number " `i'
  } 
  else { 
  drop e2_2hat
  local i = `i' + 1 
  display `i' 
  } 
     } 
  end
 
 
foreach y of varlist mmm-dis{
	
	fgls `y'_log spxt_log estimation
	predict `y'_fgls_pp_ar, resid
	gen `y'_fgls_pp_se = (e2_2hat)^0.5
	gen `y'_fgls_pp_sig = (ttail(n,abs(`y'_fgls_pp_ar/`y'_fgls_pp_se))<=.025)
	drop e2_2hat
}

** Roll FGLS 

 
foreach y of varlist mmm-dis{
      gen `y'_fgls_roll_ar = .
	  gen `y'_fgls_roll_se = .
	  gen `y'_fgls_roll_sig = .
	}	
	
forval i = `start'/`end'{
  foreach y of varlist mmm-dis{
			fgls `y'_log spxt_log date`i'
			predict ar, resid
				replace `y'_fgls_roll_ar = ar in `i'
			replace `y'_fgls_roll_se = (e2_2hat)^0.5 in `i'
			replace `y'_fgls_roll_sig = (ttail(250,abs(`y'_fgls_roll_ar/`y'_fgls_roll_se))<=.025) in `i'
			drop e2_2hat ar
			
	}
}

** pre period sq
drop estimation
gen estimation = (date >= d(24aug2006) && date <= d(08aug2007))

foreach var of varlist mmm-dis{
	
		reg `var'_log spxt_log if estimation == 1
		predict `var'_sq_pp_ar, resid
		_pctile `var'_sq_pp_ar if estimation == 1, p(2.5,97.5) altdef
		gen `var'_sq_pp_se = (abs(r(r1)) + abs(r(r2)))/2
		gen `var'_sq_pp_sig = (`var'_sq_pp_ar <= r(r1) | `var'_sq_pp_ar >= r(r2))
}

** roll sq

drop date403-date790

forval i = `start'/`end'{
	qui gen date`i' = 0
		local k = `i' - 240
		local j = `i' - 1
			forval m = `k'/`j'{
				qui replace date`i' = 1 in `m'
}
} 

foreach var of varlist mmm-dis{
		gen `var'_sq_roll_ar = .
		gen `var'_sq_roll_se = .
		gen `var'_sq_roll_sig = .
	}

forval i = `start'/`end'{
	foreach var of varlist mmm-dis{
			reg `var'_log spxt_log if date`i' == 1
			predict ar`i', resid
				replace `var'_sq_roll_ar = ar`i' in `i'
			_pctile ar`i' if date`i'== 1, p(2.5,97.5) altdef
				replace `var'_sq_roll_sig = (`var'_sq_roll_ar <= r(r1) | `var'_sq_roll_ar >= r(r2)) in `i'
				replace `var'_sq_roll_se = (abs(r(r1)) + abs(r(r2)))/2 in `i'
		drop ar`i' 	
	}
}
save full_data, replace
drop date403-date790
drop mmm-testperiod

export excel using "residual charts.xlsx" if date >= d(09aug2007) & date <= d(23feb2009), sheet("data") firstrow(variables) cell(a1) sheetmodify
